@@ -5,7 +5,7 @@
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -26,6 +26,7 @@
 #include "ginetsocketaddress.h"
 #include "ginetaddress.h"
 #include "gnetworkingprivate.h"
+#include "gsocketconnectable.h"
 #include "gioerror.h"
 #include "glibintl.h"
 
@@ -54,7 +55,13 @@ struct _GInetSocketAddressPrivate
   guint32       scope_id;
 };
 
-G_DEFINE_TYPE_WITH_PRIVATE (GInetSocketAddress, g_inet_socket_address, G_TYPE_SOCKET_ADDRESS)
+static void   g_inet_socket_address_connectable_iface_init (GSocketConnectableIface *iface);
+static gchar *g_inet_socket_address_connectable_to_string  (GSocketConnectable      *connectable);
+
+G_DEFINE_TYPE_WITH_CODE (GInetSocketAddress, g_inet_socket_address, G_TYPE_SOCKET_ADDRESS,
+                         G_ADD_PRIVATE (GInetSocketAddress)
+                         G_IMPLEMENT_INTERFACE (G_TYPE_SOCKET_CONNECTABLE,
+                                                g_inet_socket_address_connectable_iface_init))
 
 enum {
   PROP_0,
@@ -301,6 +308,59 @@ g_inet_socket_address_class_init (GInetSocketAddressClass *klass)
                                                       G_PARAM_STATIC_STRINGS));
 }
 
+static void
+g_inet_socket_address_connectable_iface_init (GSocketConnectableIface *iface)
+{
+  GSocketConnectableIface *parent_iface = g_type_interface_peek_parent (iface);
+
+  iface->enumerate = parent_iface->enumerate;
+  iface->proxy_enumerate = parent_iface->proxy_enumerate;
+  iface->to_string = g_inet_socket_address_connectable_to_string;
+}
+
+static gchar *
+g_inet_socket_address_connectable_to_string (GSocketConnectable *connectable)
+{
+  GInetSocketAddress *sa;
+  GInetAddress *a;
+  gchar *a_string;
+  GString *out;
+  guint16 port;
+
+  sa = G_INET_SOCKET_ADDRESS (connectable);
+  a = g_inet_socket_address_get_address (sa);
+  out = g_string_new ("");
+
+  /* Address. */
+  a_string = g_inet_address_to_string (a);
+  g_string_append (out, a_string);
+  g_free (a_string);
+
+  /* Scope ID (IPv6 only). */
+  if (g_inet_address_get_family (a) == G_SOCKET_FAMILY_IPV6 &&
+      g_inet_socket_address_get_scope_id (sa) != 0)
+    {
+      g_string_append_printf (out, "%%%u",
+                              g_inet_socket_address_get_scope_id (sa));
+    }
+
+  /* Port. */
+  port = g_inet_socket_address_get_port (sa);
+  if (port != 0)
+    {
+      /* Disambiguate ports from IPv6 addresses using square brackets. */
+      if (g_inet_address_get_family (a) == G_SOCKET_FAMILY_IPV6)
+        {
+          g_string_prepend (out, "[");
+          g_string_append (out, "]");
+        }
+
+      g_string_append_printf (out, ":%u", port);
+    }
+
+  return g_string_free (out, FALSE);
+}
+
 static void
 g_inet_socket_address_init (GInetSocketAddress *address)
 {
@@ -336,10 +396,10 @@ g_inet_socket_address_new (GInetAddress *address,
  * Creates a new #GInetSocketAddress for @address and @port.
  *
  * If @address is an IPv6 address, it can also contain a scope ID
- * (separated from the address by a "<literal>%</literal>").
+ * (separated from the address by a `%`).
  *
- * Returns: a new #GInetSocketAddress, or %NULL if @address cannot be
- * parsed.
+ * Returns: (nullable) (transfer full): a new #GInetSocketAddress,
+ * or %NULL if @address cannot be parsed.
  *
  * Since: 2.40
  */
@@ -361,7 +421,9 @@ g_inet_socket_address_new_from_string (const char *address,
 
       if (G_UNLIKELY (g_once_init_enter (&hints)))
         {
+          hints_struct.ai_family = AF_UNSPEC;
           hints_struct.ai_socktype = SOCK_STREAM;
+          hints_struct.ai_protocol = 0;
           hints_struct.ai_flags = AI_NUMERICHOST;
           g_once_init_leave (&hints, &hints_struct);
         }
