@@ -40,6 +40,8 @@ delete_trash_file (GFile *file, gboolean del_file, gboolean del_children)
   GFile *child;
   GFileEnumerator *enumerator;
 
+  g_return_if_fail (g_file_has_uri_scheme (file, "trash"));
+
   if (del_children)
     {
       enumerator = g_file_enumerate_children (file,
@@ -53,7 +55,14 @@ delete_trash_file (GFile *file, gboolean del_file, gboolean del_children)
           while ((info = g_file_enumerator_next_file (enumerator, NULL, NULL)) != NULL)
             {
               child = g_file_get_child (file, g_file_info_get_name (info));
-              delete_trash_file (child, TRUE, g_file_info_get_file_type (info) == G_FILE_TYPE_DIRECTORY);
+
+              /* The g_file_delete operation works differently for locations
+               * provided by the trash backend as it prevents modifications of
+               * trashed items. For that reason, it is enough to call
+               * g_file_delete on top-level items only.
+               */
+              delete_trash_file (child, TRUE, FALSE);
+
               g_object_unref (child);
               g_object_unref (info);
             }
@@ -78,7 +87,7 @@ handle_trash (int argc, char *argv[], gboolean do_help)
   g_set_prgname ("gio trash");
 
   /* Translators: commandline placeholder */
-  param = g_strdup_printf ("[%s...]", _("LOCATION"));
+  param = g_strdup_printf ("[%s…]", _("LOCATION"));
   context = g_option_context_new (param);
   g_free (param);
   g_option_context_set_help_enabled (context, FALSE);
@@ -101,8 +110,6 @@ handle_trash (int argc, char *argv[], gboolean do_help)
       return 1;
     }
 
-  g_option_context_free (context);
-
   if (argc > 1)
     {
       int i;
@@ -133,5 +140,14 @@ handle_trash (int argc, char *argv[], gboolean do_help)
       g_object_unref (file);
     }
 
+  if (argc == 1 && !empty)
+    {
+      show_help (context, _("No locations given"));
+      g_option_context_free (context);
+      return 1;
+    }
+
+  g_option_context_free (context);
+
   return retval;
 }
