@@ -51,7 +51,6 @@
 
 #include "gfileattribute.h"
 #include "glocalfile.h"
-#include "glocalfileprivate.h"
 #include "glocalfileinfo.h"
 #include "glocalfileenumerator.h"
 #include "glocalfileinputstream.h"
@@ -113,6 +112,10 @@ G_DEFINE_TYPE_WITH_CODE (GLocalFile, g_local_file, G_TYPE_OBJECT,
 
 static char *find_mountpoint_for (const char *file, dev_t dev, gboolean resolve_basename_symlink);
 
+#ifndef G_OS_WIN32
+static gboolean is_remote_fs_type (const gchar *fsname);
+#endif
+
 static void
 g_local_file_finalize (GObject *object)
 {
@@ -690,6 +693,8 @@ get_fs_type (long f_type)
       return "smackfs";
     case 0x517B:
       return "smb";
+    case 0xfe534d42:
+      return "smb2";
     case 0x534F434B:
       return "sockfs";
     case 0x73717368:
@@ -825,36 +830,6 @@ get_mount_info (GFileInfo             *fs_info,
 
 #ifdef G_OS_WIN32
 
-static gboolean
-is_xp_or_later (void)
-{
-  static int result = -1;
-
-  if (result == -1)
-    {
-#ifndef _MSC_VER    
-      OSVERSIONINFOEX ver_info = {0};
-      DWORDLONG cond_mask = 0;
-      int op = VER_GREATER_EQUAL;
-
-      ver_info.dwOSVersionInfoSize = sizeof ver_info;
-      ver_info.dwMajorVersion = 5;
-      ver_info.dwMinorVersion = 1;
-
-      VER_SET_CONDITION (cond_mask, VER_MAJORVERSION, op);
-      VER_SET_CONDITION (cond_mask, VER_MINORVERSION, op);
-
-      result = VerifyVersionInfo (&ver_info,
-				  VER_MAJORVERSION | VER_MINORVERSION, 
-				  cond_mask) != 0;
-#else
-      result = ((DWORD)(LOBYTE (LOWORD (GetVersion ())))) >= 5;  
-#endif
-    }
-
-  return result;
-}
-
 static wchar_t *
 get_volume_for_path (const char *path)
 {
@@ -913,18 +888,10 @@ get_filesystem_readonly (GFileInfo  *info,
 
   if (rootdir)
     {
-      if (is_xp_or_later ())
-        {
-          DWORD flags;
-          if (GetVolumeInformationW (rootdir, NULL, 0, NULL, NULL, &flags, NULL, 0))
-	    g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_FILESYSTEM_READONLY,
-					       (flags & FILE_READ_ONLY_VOLUME) != 0);
-        }
-      else
-        {
-          if (GetDriveTypeW (rootdir) == DRIVE_CDROM)
-	    g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_FILESYSTEM_READONLY, TRUE);
-        }
+      DWORD flags;
+      if (GetVolumeInformationW (rootdir, NULL, 0, NULL, NULL, &flags, NULL, 0))
+        g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_FILESYSTEM_READONLY,
+                                           (flags & FILE_READ_ONLY_VOLUME) != 0);
     }
 
   g_free (rootdir);
@@ -1110,11 +1077,13 @@ g_local_file_query_filesystem_info (GFile         *file,
       get_mount_info (info, local->filename, attribute_matcher);
 #endif /* G_OS_WIN32 */
     }
-  
+
+#ifndef G_OS_WIN32
   if (g_file_attribute_matcher_matches (attribute_matcher,
-					G_FILE_ATTRIBUTE_FILESYSTEM_REMOTE))
-      g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_FILESYSTEM_REMOTE,
-					 g_local_file_is_remote (local->filename));
+                                        G_FILE_ATTRIBUTE_FILESYSTEM_REMOTE))
+    g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_FILESYSTEM_REMOTE,
+                                       is_remote_fs_type (fstype));
+#endif
 
   g_file_attribute_matcher_unref (attribute_matcher);
   
@@ -1383,13 +1352,9 @@ g_local_file_read (GFile         *file,
       return NULL;
     }
 
-#ifdef G_OS_WIN32
-  ret = GLIB_PRIVATE_CALL (g_win32_fstat) (fd, &buf);
-#else
-  ret = fstat (fd, &buf);
-#endif
+  ret = g_local_file_fstat (fd, G_LOCAL_FILE_STAT_FIELD_TYPE, G_LOCAL_FILE_STAT_FIELD_ALL, &buf);
 
-  if (ret == 0 && S_ISDIR (buf.st_mode))
+  if (ret == 0 && S_ISDIR (_g_stat_mode (&buf)))
     {
       (void) g_close (fd, NULL);
       g_set_io_error (error,
@@ -1808,6 +1773,52 @@ try_make_relative (const char *path,
   return g_strdup (path);
 }
 
+static gboolean
+ignore_trash_mount (GUnixMountEntry *mount)
+{
+  GUnixMountPoint *mount_point = NULL;
+  const gchar *mount_options;
+  gboolean retval = TRUE;
+
+  if (g_unix_mount_is_system_internal (mount))
+    return TRUE;
+
+  mount_options = g_unix_mount_get_options (mount);
+  if (mount_options == NULL)
+    {
+      mount_point = g_unix_mount_point_at (g_unix_mount_get_mount_path (mount),
+                                           NULL);
+      if (mount_point != NULL)
+        mount_options = g_unix_mount_point_get_options (mount_point);
+    }
+
+  if (mount_options == NULL ||
+      strstr (mount_options, "x-gvfs-notrash") == NULL)
+    retval = FALSE;
+
+  g_clear_pointer (&mount_point, g_unix_mount_point_free);
+
+  return retval;
+}
+
+static gboolean
+ignore_trash_path (const gchar *topdir)
+{
+  GUnixMountEntry *mount;
+  gboolean retval = TRUE;
+
+  mount = g_unix_mount_at (topdir, NULL);
+  if (mount == NULL)
+    goto out;
+
+  retval = ignore_trash_mount (mount);
+
+ out:
+  g_clear_pointer (&mount, g_unix_mount_free);
+
+  return retval;
+}
+
 gboolean
 _g_local_file_has_trash_dir (const char *dirname, dev_t dir_dev)
 {
@@ -1818,7 +1829,6 @@ _g_local_file_has_trash_dir (const char *dirname, dev_t dir_dev)
   char uid_str[32];
   GStatBuf global_stat, trash_stat;
   gboolean res;
-  GUnixMountEntry *mount;
 
   if (g_once_init_enter (&home_dev_set))
     {
@@ -1837,17 +1847,13 @@ _g_local_file_has_trash_dir (const char *dirname, dev_t dir_dev)
   if (topdir == NULL)
     return FALSE;
 
-  mount = g_unix_mount_at (topdir, NULL);
-  if (mount == NULL || g_unix_mount_is_system_internal (mount))
+  if (ignore_trash_path (topdir))
     {
-      g_clear_pointer (&mount, g_unix_mount_free);
       g_free (topdir);
 
       return FALSE;
     }
 
-  g_clear_pointer (&mount, g_unix_mount_free);
-
   globaldir = g_build_filename (topdir, ".Trash", NULL);
   if (g_lstat (globaldir, &global_stat) == 0 &&
       S_ISDIR (global_stat.st_mode) &&
@@ -2013,7 +2019,6 @@ g_local_file_trash (GFile         *file,
     {
       uid_t uid;
       char uid_str[32];
-      GUnixMountEntry *mount;
 
       uid = geteuid ();
       g_snprintf (uid_str, sizeof (uid_str), "%lu", (unsigned long)uid);
@@ -2027,20 +2032,15 @@ g_local_file_trash (GFile         *file,
 	  return FALSE;
 	}
 
-      mount = g_unix_mount_at (topdir, NULL);
-      if (mount == NULL || g_unix_mount_is_system_internal (mount))
+      if (ignore_trash_path (topdir))
         {
           g_set_error (error, G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED,
                        _("Trashing on system internal mounts is not supported"));
-
-          g_clear_pointer (&mount, g_unix_mount_free);
           g_free (topdir);
 
           return FALSE;
         }
 
-      g_clear_pointer (&mount, g_unix_mount_free);
-
       /* Try looking for global trash dir $topdir/.Trash/$uid */
       globaldir = g_build_filename (topdir, ".Trash", NULL);
       if (g_lstat (globaldir, &global_stat) == 0 &&
@@ -2204,7 +2204,9 @@ g_local_file_trash (GFile         *file,
 			  original_name_escaped, delete_time);
   g_free (delete_time);
 
-  g_file_set_contents (infofile, data, -1, NULL);
+  g_file_set_contents_full (infofile, data, -1,
+                            G_FILE_SET_CONTENTS_CONSISTENT | G_FILE_SET_CONTENTS_ONLY_EXISTING,
+                            0600, NULL);
 
   /* TODO: Maybe we should verify that you can delete the file from the trash
    * before moving it? OTOH, that is hard, as it needs a recursive scan
@@ -2509,7 +2511,7 @@ g_local_file_move (GFile                  *source,
 #ifdef G_OS_WIN32
 
 gboolean
-g_local_file_is_remote (const gchar *filename)
+g_local_file_is_nfs_home (const gchar *filename)
 {
   return FALSE;
 }
@@ -2517,58 +2519,29 @@ g_local_file_is_remote (const gchar *filename)
 #else
 
 static gboolean
-is_remote_fs (const gchar *filename)
+is_remote_fs_type (const gchar *fsname)
 {
-  const char *fsname = NULL;
-
-#ifdef USE_STATFS
-  struct statfs statfs_buffer;
-  int statfs_result = 0;
-
-#if STATFS_ARGS == 2
-  statfs_result = statfs (filename, &statfs_buffer);
-#elif STATFS_ARGS == 4
-  statfs_result = statfs (filename, &statfs_buffer, sizeof (statfs_buffer), 0);
-#endif
-
-#elif defined(USE_STATVFS)
-  struct statvfs statfs_buffer;
-  int statfs_result = 0;
-
-  statfs_result = statvfs (filename, &statfs_buffer);
-#else
-  return FALSE;
-#endif
-
-  if (statfs_result == -1)
-    return FALSE;
-
-#ifdef USE_STATFS
-#if defined(HAVE_STRUCT_STATFS_F_FSTYPENAME)
-  fsname = statfs_buffer.f_fstypename;
-#else
-  fsname = get_fs_type (statfs_buffer.f_type);
-#endif
-
-#elif defined(USE_STATVFS) && defined(HAVE_STRUCT_STATVFS_F_BASETYPE)
-  fsname = statfs_buffer.f_basetype;
-#endif
-
   if (fsname != NULL)
     {
       if (strcmp (fsname, "nfs") == 0)
         return TRUE;
       if (strcmp (fsname, "nfs4") == 0)
         return TRUE;
+      if (strcmp (fsname, "cifs") == 0)
+        return TRUE;
+      if (strcmp (fsname, "smb") == 0)
+        return TRUE;
+      if (strcmp (fsname, "smb2") == 0)
+        return TRUE;
     }
 
   return FALSE;
 }
 
 gboolean
-g_local_file_is_remote (const gchar *filename)
+g_local_file_is_nfs_home (const gchar *filename)
 {
-  static gboolean remote_home;
+  static gboolean remote_home = FALSE;
   static gsize initialized;
   const gchar *home;
 
@@ -2577,7 +2550,19 @@ g_local_file_is_remote (const gchar *filename)
     {
       if (g_once_init_enter (&initialized))
         {
-          remote_home = is_remote_fs (home);
+          GFile *file;
+          GFileInfo *info;
+          const gchar *fs_type = NULL;
+
+          file = _g_local_file_new (home);
+          info = g_local_file_query_filesystem_info (file, G_FILE_ATTRIBUTE_FILESYSTEM_TYPE, NULL, NULL);
+          if (info != NULL)
+            fs_type = g_file_info_get_attribute_string (info, G_FILE_ATTRIBUTE_FILESYSTEM_TYPE);
+          if (g_strcmp0 (fs_type, "nfs") == 0 || g_strcmp0 (fs_type, "nfs4") == 0)
+            remote_home = TRUE;
+          g_clear_object (&info);
+          g_object_unref (file);
+
           g_once_init_leave (&initialized, TRUE);
         }
       return remote_home;
@@ -2717,7 +2702,10 @@ g_local_file_measure_size_of_file (gint           parent_fd,
     return FALSE;
 
 #if defined (AT_FDCWD)
-  if (fstatat (parent_fd, name->data, &buf, AT_SYMLINK_NOFOLLOW) != 0)
+  if (g_local_file_fstatat (parent_fd, name->data, AT_SYMLINK_NOFOLLOW,
+                            G_LOCAL_FILE_STAT_FIELD_BASIC_STATS,
+                            G_LOCAL_FILE_STAT_FIELD_ALL & (~G_LOCAL_FILE_STAT_FIELD_ATIME),
+                            &buf) != 0)
     {
       int errsv = errno;
       return g_local_file_measure_size_error (state->flags, errsv, name, error);
@@ -2741,7 +2729,7 @@ g_local_file_measure_size_of_file (gint           parent_fd,
       /* If not at the toplevel, check for a device boundary. */
 
       if (state->flags & G_FILE_MEASURE_NO_XDEV)
-        if (state->contained_on != buf.st_dev)
+        if (state->contained_on != _g_stat_dev (&buf))
           return TRUE;
     }
   else
@@ -2749,7 +2737,7 @@ g_local_file_measure_size_of_file (gint           parent_fd,
       /* If, however, this is the toplevel, set the device number so
        * that recursive invocations can compare against it.
        */
-      state->contained_on = buf.st_dev;
+      state->contained_on = _g_stat_dev (&buf);
     }
 
 #if defined (G_OS_WIN32)
@@ -2758,12 +2746,12 @@ g_local_file_measure_size_of_file (gint           parent_fd,
   else
 #elif defined (HAVE_STRUCT_STAT_ST_BLOCKS)
   if (~state->flags & G_FILE_MEASURE_APPARENT_SIZE)
-    state->disk_usage += buf.st_blocks * G_GUINT64_CONSTANT (512);
+    state->disk_usage += _g_stat_blocks (&buf) * G_GUINT64_CONSTANT (512);
   else
 #endif
-    state->disk_usage += buf.st_size;
+    state->disk_usage += _g_stat_size (&buf);
 
-  if (S_ISDIR (buf.st_mode))
+  if (S_ISDIR (_g_stat_mode (&buf)))
     state->num_dirs++;
   else
     state->num_files++;
@@ -2798,7 +2786,7 @@ g_local_file_measure_size_of_file (gint           parent_fd,
         }
     }
 
-  if (S_ISDIR (buf.st_mode))
+  if (S_ISDIR (_g_stat_mode (&buf)))
     {
       int dir_fd = -1;
 #ifdef AT_FDCWD
