@@ -7,7 +7,7 @@
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -302,13 +302,16 @@ eject_unmount_do (GMount              *mount,
                   GCancellable        *cancellable,
                   GAsyncReadyCallback  callback,
                   gpointer             user_data,
-                  char               **argv)
+                  char               **argv,
+                  const gchar         *task_name)
 {
   GUnixMount *unix_mount = G_UNIX_MOUNT (mount);
   GTask *task;
   GSource *timeout;
 
   task = g_task_new (mount, cancellable, callback, user_data);
+  g_task_set_source_tag (task, eject_unmount_do);
+  g_task_set_name (task, task_name);
   g_task_set_task_data (task, g_strdupv (argv), (GDestroyNotify) g_strfreev);
 
   if (unix_mount->volume_monitor != NULL)
@@ -336,7 +339,7 @@ g_unix_mount_unmount (GMount             *mount,
   else
     argv[1] = unix_mount->device_path;
 
-  eject_unmount_do (mount, cancellable, callback, user_data, argv);
+  eject_unmount_do (mount, cancellable, callback, user_data, argv, "[gio] unmount mount");
 }
 
 static gboolean
@@ -362,7 +365,7 @@ g_unix_mount_eject (GMount             *mount,
   else
     argv[1] = unix_mount->device_path;
 
-  eject_unmount_do (mount, cancellable, callback, user_data, argv);
+  eject_unmount_do (mount, cancellable, callback, user_data, argv, "[gio] eject mount");
 }
 
 static gboolean
