@@ -5,7 +5,7 @@
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -239,8 +239,9 @@ gboolean         g_win32_registry_value_iter_get_data_w      (GWin32RegistryValu
                                                               gsize                           *value_data_size,
                                                               GError                         **error);
 
-GLIB_AVAILABLE_IN_2_46
+GLIB_AVAILABLE_IN_2_66
 gboolean         g_win32_registry_key_get_value              (GWin32RegistryKey               *key,
+                                                              const gchar * const             *mui_dll_dirs,
                                                               gboolean                         auto_expand,
                                                               const gchar                     *value_name,
                                                               GWin32RegistryValueType         *value_type,
@@ -248,8 +249,9 @@ gboolean         g_win32_registry_key_get_value              (GWin32RegistryKey
                                                               gsize                           *value_data_size,
                                                               GError                         **error);
 
-GLIB_AVAILABLE_IN_2_46
+GLIB_AVAILABLE_IN_2_66
 gboolean         g_win32_registry_key_get_value_w            (GWin32RegistryKey               *key,
+                                                              const gunichar2 * const         *mui_dll_dirs,
                                                               gboolean                         auto_expand,
                                                               const gunichar2                 *value_name,
                                                               GWin32RegistryValueType         *value_type,
@@ -276,6 +278,12 @@ gboolean         g_win32_registry_key_has_changed            (GWin32RegistryKey
 GLIB_AVAILABLE_IN_2_46
 void             g_win32_registry_key_erase_change_indicator (GWin32RegistryKey               *key);
 
+GLIB_AVAILABLE_IN_2_66
+const gunichar2 * const *g_win32_registry_get_os_dirs_w (void);
+
+GLIB_AVAILABLE_IN_2_66
+const gchar * const     *g_win32_registry_get_os_dirs   (void);
+
 G_END_DECLS
 
 #endif /* G_PLATFORM_WIN32 */
