@@ -1,11 +1,33 @@
 #include <gio/gio.h>
 #include <gstdio.h>
 
+typedef struct
+{
+  gchar *applications_dir;
+} Fixture;
+
+static void
+setup (Fixture       *fixture,
+       gconstpointer  user_data)
+{
+  fixture->applications_dir = g_build_filename (g_get_user_data_dir (), "applications", NULL);
+  g_assert_cmpint (g_mkdir_with_parents (fixture->applications_dir, 0755), ==, 0);
+
+  g_test_message ("Using data directory: %s", g_get_user_data_dir ());
+}
+
+static void
+teardown (Fixture       *fixture,
+          gconstpointer  user_data)
+{
+  g_assert_cmpint (g_rmdir (fixture->applications_dir), ==, 0);
+  g_clear_pointer (&fixture->applications_dir, g_free);
+}
+
 static gboolean
 create_app (gpointer data)
 {
   const gchar *path = data;
-  gchar *file;
   GError *error = NULL;
   const gchar *contents = 
     "[Desktop Entry]\n"
@@ -14,13 +36,9 @@ create_app (gpointer data)
     "Type=Application\n"
     "Exec=true\n";
 
-  file = g_build_filename (path, "app.desktop", NULL);
-
-  g_file_set_contents (file, contents, -1, &error);
+  g_file_set_contents (path, contents, -1, &error);
   g_assert_no_error (error);
 
-  g_free (file);
-
   return G_SOURCE_REMOVE;
 }
 
@@ -28,13 +46,8 @@ static void
 delete_app (gpointer data)
 {
   const gchar *path = data;
-  gchar *file;
 
-  file = g_build_filename (path, "app.desktop", NULL);
-
-  g_remove (file);
-
-  g_free (file);
+  g_remove (path);
 }
 
 static gboolean changed_fired;
@@ -58,14 +71,14 @@ quit_loop (gpointer data)
 }
 
 static void
-test_app_monitor (void)
+test_app_monitor (Fixture       *fixture,
+                  gconstpointer  user_data)
 {
-  gchar *path;
+  gchar *app_path;
   GAppInfoMonitor *monitor;
   GMainLoop *loop;
 
-  path = g_build_filename (g_get_user_data_dir (), "applications", NULL);
-  g_mkdir (path, 0755);
+  app_path = g_build_filename (fixture->applications_dir, "app.desktop", NULL);
 
   /* FIXME: this shouldn't be required */
   g_list_free_full (g_app_info_get_all (), g_object_unref);
@@ -75,7 +88,7 @@ test_app_monitor (void)
 
   g_signal_connect (monitor, "changed", G_CALLBACK (changed_cb), loop);
 
-  g_idle_add (create_app, path);
+  g_idle_add (create_app, app_path);
   g_timeout_add_seconds (3, quit_loop, loop);
 
   g_main_loop_run (loop);
@@ -87,31 +100,26 @@ test_app_monitor (void)
 
   g_timeout_add_seconds (3, quit_loop, loop);
 
-  delete_app (path);
+  delete_app (app_path);
 
   g_main_loop_run (loop);
 
   g_assert (changed_fired);
 
   g_main_loop_unref (loop);
+  g_remove (app_path);
 
   g_object_unref (monitor);
 
-  g_free (path);
+  g_free (app_path);
 }
 
 int
 main (int argc, char *argv[])
 {
-  gchar *path;
+  g_test_init (&argc, &argv, G_TEST_OPTION_ISOLATE_DIRS, NULL);
 
-  path = g_mkdtemp (g_strdup ("app_monitor_XXXXXX"));
-  g_setenv ("XDG_DATA_DIRS", path, TRUE);
-  g_setenv ("XDG_DATA_HOME", path, TRUE);
-
-  g_test_init (&argc, &argv, NULL);
-
-  g_test_add_func ("/monitor/app", test_app_monitor);
+  g_test_add ("/monitor/app", Fixture, NULL, setup, test_app_monitor, teardown);
 
   return g_test_run ();
 }
