@@ -244,6 +244,11 @@ assert_expected_uid_pid (InteropFlags flags,
        * on Linux. */
       g_assert_cmpint (uid, ==, getuid ());
       g_assert_cmpint (pid, ==, getpid ());
+#elif defined(__APPLE__)
+      /* We know (or at least suspect) that both GDBus and libdbus support
+       * passing the uid only on macOS. */
+      g_assert_cmpint (uid, ==, getuid ());
+      /* No pid here */
 #else
       g_test_message ("Please open a merge request to add appropriate "
                       "assertions for your platform");
