@@ -1116,11 +1116,12 @@ g_private_replace (GPrivate *key,
   gint status;
 
   old = pthread_getspecific (*impl);
-  if (old && key->notify)
-    key->notify (old);
 
   if G_UNLIKELY ((status = pthread_setspecific (*impl, value)) != 0)
     g_thread_abort (status, "pthread_setspecific");
+
+  if (old && key->notify)
+    key->notify (old);
 }
 
 /* {{{1 GThread */
@@ -1235,6 +1236,7 @@ static void *
 linux_pthread_proxy (void *data)
 {
   GThreadPosix *thread = data;
+  static gboolean printed_scheduler_warning = FALSE;  /* (atomic) */
 
   /* Set scheduler settings first if requested */
   if (thread->scheduler_settings)
@@ -1247,8 +1249,11 @@ linux_pthread_proxy (void *data)
       tid = (pid_t) syscall (SYS_gettid);
       res = syscall (SYS_sched_setattr, tid, thread->scheduler_settings->attr, flags);
       errsv = errno;
-      if (res == -1)
+      if (res == -1 && g_atomic_int_compare_and_exchange (&printed_scheduler_warning, FALSE, TRUE))
         g_critical ("Failed to set scheduler settings: %s", g_strerror (errsv));
+      else if (res == -1)
+        g_debug ("Failed to set scheduler settings: %s", g_strerror (errsv));
+      printed_scheduler_warning = TRUE;
     }
 
   return thread->proxy (data);
