@@ -28,9 +28,10 @@ test_load_from_data_dirs (void)
 
   res = g_bookmark_file_load_from_data_dirs (bookmark, "no-such-bookmark-file.xbel", &path, &error);
 
-  g_assert (!res);
+  g_assert_false (res);
   g_assert_error (error, G_FILE_ERROR, G_FILE_ERROR_NOENT);
   g_assert_null (path);
+  g_error_free (error);
 
   g_bookmark_file_free (bookmark);  
 }
@@ -42,26 +43,53 @@ test_to_file (void)
   const gchar *filename;
   gboolean res;
   GError *error = NULL;
-  gchar *in, *out;
+  char *in, *out;
 
   bookmark = g_bookmark_file_new ();
 
-  filename = g_test_get_filename (G_TEST_DIST, "bookmarks", "valid-01.xbel", NULL);
-  res = g_bookmark_file_load_from_file (bookmark, filename, &error);
-  g_assert (res);
-  g_assert_no_error (error);
+  g_test_message ("Roundtrip from newly created bookmark file");
+  g_bookmark_file_set_title (bookmark, "file:///tmp/schedule.ps", "schedule.ps");
+  g_bookmark_file_set_mime_type (bookmark, "file:///tmp/schedule.ps", "application/postscript");
+  g_bookmark_file_add_application (bookmark, "file:///tmp/schedule.ps", "ghostscript", "ghostscript %F");
 
   res = g_bookmark_file_to_file (bookmark, "out.xbel", &error);
-  g_assert (res);
   g_assert_no_error (error);
+  g_assert_true (res);
+
+  res = g_bookmark_file_load_from_file (bookmark, "out.xbel", &error);
+  g_assert_no_error (error);
+  g_assert_true (res);
+
+  out = g_bookmark_file_get_title (bookmark, "file:///tmp/schedule.ps", &error);
+  g_assert_no_error (error);
+  g_assert_cmpstr (out, ==, "schedule.ps");
+  g_free (out);
+
+  out = g_bookmark_file_get_mime_type (bookmark, "file:///tmp/schedule.ps", &error);
+  g_assert_no_error (error);
+  g_assert_cmpstr (out, ==, "application/postscript");
+  g_free (out);
+
+  remove ("out.xbel");
+
+  g_test_message ("Roundtrip from a valid bookmark file");
+  filename = g_test_get_filename (G_TEST_DIST, "bookmarks", "valid-01.xbel", NULL);
+  res = g_bookmark_file_load_from_file (bookmark, filename, &error);
+  g_assert_no_error (error);
+  g_assert_true (res);
+
+  res = g_bookmark_file_to_file (bookmark, "out.xbel", &error);
+  g_assert_no_error (error);
+  g_assert_true (res);
 
   res = g_file_get_contents (filename, &in, NULL, &error);
-  g_assert (res);
   g_assert_no_error (error);
+  g_assert_true (res);
 
   res = g_file_get_contents ("out.xbel", &out, NULL, &error);
-  g_assert (res);
   g_assert_no_error (error);
+  g_assert_true (res);
+  remove ("out.xbel");
 
   g_assert_cmpstr (in, ==, out);
   g_free (in);
@@ -82,21 +110,28 @@ test_move_item (void)
 
   filename = g_test_get_filename (G_TEST_DIST, "bookmarks", "valid-01.xbel", NULL);
   res = g_bookmark_file_load_from_file (bookmark, filename, &error);
-  g_assert (res);
+  g_assert_true (res);
   g_assert_no_error (error);
 
   res = g_bookmark_file_move_item (bookmark,
                                    "file:///home/zefram/Documents/milan-stuttgart.ps",
                                    "file:///tmp/schedule.ps",
                                    &error);
-  g_assert (res);
+  g_assert_true (res);
+  g_assert_no_error (error);
+
+  res = g_bookmark_file_move_item (bookmark,
+                                   "file:///tmp/schedule.ps",
+                                   "file:///tmp/schedule.ps",
+                                   &error);
+  g_assert_true (res);
   g_assert_no_error (error);
 
   res = g_bookmark_file_move_item (bookmark,
                                    "file:///no-such-file.xbel",
                                    "file:///tmp/schedule.ps",
                                    &error);
-  g_assert (!res);
+  g_assert_false (res);
   g_assert_error (error, G_BOOKMARK_FILE_ERROR, G_BOOKMARK_FILE_ERROR_URI_NOT_FOUND);
   g_clear_error (&error);
 
@@ -104,7 +139,7 @@ test_move_item (void)
                                    "file:///tmp/schedule.ps",
                                    NULL,
                                    &error);
-  g_assert (res);
+  g_assert_true (res);
   g_assert_no_error (error);
 
   g_bookmark_file_free (bookmark);
@@ -118,7 +153,7 @@ test_misc (void)
   gboolean res;
   GError *error = NULL;
   gchar *s;
-  time_t now, t;
+  GDateTime *before, *after, *t;
   gchar *cmd, *exec;
   guint count;
 
@@ -126,7 +161,7 @@ test_misc (void)
 
   filename = g_test_get_filename (G_TEST_DIST, "bookmarks", "valid-01.xbel", NULL);
   res = g_bookmark_file_load_from_file (bookmark, filename, &error);
-  g_assert (res);
+  g_assert_true (res);
   g_assert_no_error (error);
 
   res = g_bookmark_file_get_icon (bookmark,
@@ -134,7 +169,7 @@ test_misc (void)
                                   NULL,
                                   NULL,
                                   &error);
-  g_assert (!res);
+  g_assert_false (res);
   g_assert_no_error (error);
 
   res = g_bookmark_file_get_icon (bookmark,
@@ -142,7 +177,7 @@ test_misc (void)
                                   NULL,
                                   NULL,
                                   &error);
-  g_assert (!res);
+  g_assert_false (res);
   g_assert_error (error, G_BOOKMARK_FILE_ERROR, G_BOOKMARK_FILE_ERROR_URI_NOT_FOUND);
   g_clear_error (&error);
 
@@ -166,6 +201,7 @@ test_misc (void)
                                         &error);
   g_assert_error (error, G_BOOKMARK_FILE_ERROR, G_BOOKMARK_FILE_ERROR_INVALID_VALUE);
   g_clear_error (&error);
+  g_assert_false (res);
 
   g_bookmark_file_set_mime_type (bookmark, 
                                  "file:///tmp/schedule1.ps",
@@ -184,35 +220,57 @@ test_misc (void)
                                         "file:///tmp/schedule2.ps",
                                         &error);
   g_assert_no_error (error);
-  g_assert (res);
+  g_assert_true (res);
 
-  time (&now);
-  g_bookmark_file_set_added (bookmark,
-                             "file:///tmp/schedule3.ps",
-                             (time_t)-1);
-  t = g_bookmark_file_get_added (bookmark,
-                                 "file:///tmp/schedule3.ps",
-                                 &error);
-  g_assert_no_error (error);
-  g_assert (t == now);
+  before = g_date_time_new_now_utc ();
 
-  g_bookmark_file_set_modified (bookmark,
-                                "file:///tmp/schedule4.ps",
-                                (time_t)-1);
-  t = g_bookmark_file_get_modified (bookmark,
-                                    "file:///tmp/schedule4.ps",
-                                    &error);
+  g_bookmark_file_set_added_date_time (bookmark,
+                                       "file:///tmp/schedule3.ps",
+                                       before);
+  t = g_bookmark_file_get_added_date_time (bookmark,
+                                           "file:///tmp/schedule3.ps",
+                                           &error);
   g_assert_no_error (error);
-  g_assert (t == now);
 
-  g_bookmark_file_set_visited (bookmark,
-                               "file:///tmp/schedule5.ps",
-                               (time_t)-1);
-  t = g_bookmark_file_get_visited (bookmark,
-                                   "file:///tmp/schedule5.ps",
-                                   &error);
+  after = g_date_time_new_now_utc ();
+  g_assert_cmpint (g_date_time_compare (before, t), <=, 0);
+  g_assert_cmpint (g_date_time_compare (t, after), <=, 0);
+
+  g_date_time_unref (after);
+  g_date_time_unref (before);
+
+  before = g_date_time_new_now_utc ();
+
+  g_bookmark_file_set_modified_date_time (bookmark,
+                                          "file:///tmp/schedule4.ps",
+                                          before);
+  t = g_bookmark_file_get_modified_date_time (bookmark,
+                                              "file:///tmp/schedule4.ps",
+                                              &error);
   g_assert_no_error (error);
-  g_assert (t == now);
+
+  after = g_date_time_new_now_utc ();
+  g_assert_cmpint (g_date_time_compare (before, t), <=, 0);
+  g_assert_cmpint (g_date_time_compare (t, after), <=, 0);
+
+  g_date_time_unref (after);
+  g_date_time_unref (before);
+
+  before = g_date_time_new_now_utc ();
+
+  g_bookmark_file_set_visited_date_time (bookmark,
+                                         "file:///tmp/schedule5.ps",
+                                         before);
+  t = g_bookmark_file_get_visited_date_time (bookmark,
+                                             "file:///tmp/schedule5.ps",
+                                             &error);
+  g_assert_no_error (error);
+
+  after = g_date_time_new_now_utc ();
+  g_assert_cmpint (g_date_time_compare (before, t), <=, 0);
+  g_assert_cmpint (g_date_time_compare (t, after), <=, 0);
+  g_date_time_unref (after);
+  g_date_time_unref (before);
 
   g_bookmark_file_set_icon (bookmark,
                             "file:///tmp/schedule6.ps",
@@ -224,7 +282,7 @@ test_misc (void)
                                   NULL, 
                                   &error);
   g_assert_no_error (error);
-  g_assert (res);
+  g_assert_true (res);
   g_assert_cmpstr (s, ==, "application-x-postscript");
   g_free (s);
 
@@ -237,36 +295,138 @@ test_misc (void)
                                   NULL, 
                                   &error);
   g_assert_no_error (error);
-  g_assert (!res);
+  g_assert_false (res);
 
   res = g_bookmark_file_has_application (bookmark,
                                          "file:///tmp/schedule7.ps",
                                          "foo",
                                          &error);
   g_assert_error (error, G_BOOKMARK_FILE_ERROR, G_BOOKMARK_FILE_ERROR_URI_NOT_FOUND);
-  g_assert (!res);
+  g_assert_false (res);
   g_clear_error (&error);
 
-  g_bookmark_file_add_application (bookmark, 
+  before = g_date_time_new_now_utc ();
+
+  g_bookmark_file_add_application (bookmark,
                                    "file:///tmp/schedule7.ps",
                                    NULL, NULL);
-  res = g_bookmark_file_get_app_info (bookmark,
-                                      "file:///tmp/schedule7.ps",
-                                      g_get_application_name (),
-                                      &exec, &count, &t,
-                                      &error);
+  res = g_bookmark_file_get_application_info (bookmark,
+                                              "file:///tmp/schedule7.ps",
+                                              g_get_application_name (),
+                                              &exec, &count, &t,
+                                              &error);
   g_assert_no_error (error);
-  g_assert (res);
+  g_assert_true (res);
   cmd = g_strconcat (g_get_prgname (), " file:///tmp/schedule7.ps", NULL);
   g_assert_cmpstr (exec, ==, cmd);
   g_free (cmd);
   g_free (exec);
   g_assert_cmpuint (count, ==, 1);
-  g_assert (t == now);
+
+  after = g_date_time_new_now_utc ();
+  g_assert_cmpint (g_date_time_compare (before, t), <=, 0);
+  g_assert_cmpint (g_date_time_compare (t, after), <=, 0);
+
+  g_date_time_unref (after);
+  g_date_time_unref (before);
 
   g_bookmark_file_free (bookmark);
 }
 
+static void
+test_deprecated (void)
+{
+  GBookmarkFile *file = NULL;
+  GError *local_error = NULL;
+  time_t t, now;
+  gboolean retval;
+
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
+
+  now = time (NULL);
+  file = g_bookmark_file_new ();
+
+  /* added */
+  g_bookmark_file_set_added (file, "file://test", -1);
+  t = g_bookmark_file_get_added (file, "file://test", &local_error);
+  g_assert_no_error (local_error);
+  g_assert_cmpint (t, >=, now);
+
+  g_bookmark_file_set_added (file, "file://test", 1234);
+  t = g_bookmark_file_get_added (file, "file://test", &local_error);
+  g_assert_no_error (local_error);
+  g_assert_cmpint (t, ==, 1234);
+
+  t = g_bookmark_file_get_added (file, "file://not-exist", &local_error);
+  g_assert_error (local_error, G_BOOKMARK_FILE_ERROR, G_BOOKMARK_FILE_ERROR_URI_NOT_FOUND);
+  g_assert_cmpint (t, ==, (time_t) -1);
+  g_clear_error (&local_error);
+
+  /* modified */
+  g_bookmark_file_set_modified (file, "file://test", -1);
+  t = g_bookmark_file_get_modified (file, "file://test", &local_error);
+  g_assert_no_error (local_error);
+  g_assert_cmpint (t, >=, now);
+
+  g_bookmark_file_set_modified (file, "file://test", 1234);
+  t = g_bookmark_file_get_modified (file, "file://test", &local_error);
+  g_assert_no_error (local_error);
+  g_assert_cmpint (t, ==, 1234);
+
+  t = g_bookmark_file_get_modified (file, "file://not-exist", &local_error);
+  g_assert_error (local_error, G_BOOKMARK_FILE_ERROR, G_BOOKMARK_FILE_ERROR_URI_NOT_FOUND);
+  g_assert_cmpint (t, ==, (time_t) -1);
+  g_clear_error (&local_error);
+
+  /* visited */
+  g_bookmark_file_set_visited (file, "file://test", -1);
+  t = g_bookmark_file_get_visited (file, "file://test", &local_error);
+  g_assert_no_error (local_error);
+  g_assert_cmpint (t, >=, now);
+
+  g_bookmark_file_set_visited (file, "file://test", 1234);
+  t = g_bookmark_file_get_visited (file, "file://test", &local_error);
+  g_assert_no_error (local_error);
+  g_assert_cmpint (t, ==, 1234);
+
+  t = g_bookmark_file_get_visited (file, "file://not-exist", &local_error);
+  g_assert_error (local_error, G_BOOKMARK_FILE_ERROR, G_BOOKMARK_FILE_ERROR_URI_NOT_FOUND);
+  g_assert_cmpint (t, ==, (time_t) -1);
+  g_clear_error (&local_error);
+
+  /* set app info */
+  retval = g_bookmark_file_set_app_info (file, "file://test", "app", "/path/to/app", 1, -1, &local_error);
+  g_assert_no_error (local_error);
+  g_assert_true (retval);
+
+  retval = g_bookmark_file_get_app_info (file, "file://test", "app", NULL, NULL, &t, &local_error);
+  g_assert_no_error (local_error);
+  g_assert_true (retval);
+  g_assert_cmpint (t, >=, now);
+
+  retval = g_bookmark_file_set_app_info (file, "file://test", "app", "/path/to/app", 1, 1234, &local_error);
+  g_assert_no_error (local_error);
+  g_assert_true (retval);
+
+  retval = g_bookmark_file_get_app_info (file, "file://test", "app", NULL, NULL, &t, &local_error);
+  g_assert_no_error (local_error);
+  g_assert_true (retval);
+  g_assert_cmpint (t, ==, 1234);
+
+  retval = g_bookmark_file_get_app_info (file, "file://test", "app", NULL, NULL, NULL, &local_error);
+  g_assert_no_error (local_error);
+  g_assert_true (retval);
+
+  retval = g_bookmark_file_get_app_info (file, "file://not-exist", "app", NULL, NULL, &t, &local_error);
+  g_assert_error (local_error, G_BOOKMARK_FILE_ERROR, G_BOOKMARK_FILE_ERROR_URI_NOT_FOUND);
+  g_assert_false (retval);
+  g_clear_error (&local_error);
+
+  g_bookmark_file_free (file);
+
+G_GNUC_END_IGNORE_DEPRECATIONS
+}
+
 static gboolean
 test_load (GBookmarkFile *bookmark,
            const gchar   *filename)
@@ -276,7 +436,7 @@ test_load (GBookmarkFile *bookmark,
   
   res = g_bookmark_file_load_from_file (bookmark, filename, &error);
   if (error && g_test_verbose ())
-    g_print ("Load error: %s\n", error->message);
+    g_printerr ("Load error: %s\n", error->message);
 
   g_clear_error (&error);
   return res;
@@ -298,19 +458,19 @@ test_query (GBookmarkFile *bookmark)
 
   for (i = 0; i < uris_len; i++)
     {
-      g_assert (g_bookmark_file_has_item (bookmark, uris[i]));
+      g_assert_true (g_bookmark_file_has_item (bookmark, uris[i]));
       error = NULL;
       mime = g_bookmark_file_get_mime_type (bookmark, uris[i], &error);
-      g_assert (mime != NULL);
+      g_assert_nonnull (mime);
       g_assert_no_error (error);
       g_free (mime);
     }
   g_strfreev (uris);
 
-  g_assert (!g_bookmark_file_has_item (bookmark, "file:///no/such/uri"));
+  g_assert_false (g_bookmark_file_has_item (bookmark, "file:///no/such/uri"));
   error = NULL;
   mime = g_bookmark_file_get_mime_type (bookmark, "file:///no/such/uri", &error);
-  g_assert (mime == NULL);
+  g_assert_null (mime);
   g_assert_error (error, G_BOOKMARK_FILE_ERROR, G_BOOKMARK_FILE_ERROR_URI_NOT_FOUND);
   g_error_free (error);
   g_free (mime);
@@ -321,8 +481,8 @@ test_modify (GBookmarkFile *bookmark)
 {
   gchar *text;
   guint count;
-  time_t stamp;
-  time_t now;
+  GDateTime *stamp;
+  GDateTime *now = NULL;
   GError *error = NULL;
   gchar **groups;
   gsize length;
@@ -331,7 +491,7 @@ test_modify (GBookmarkFile *bookmark)
   gchar *mime;
 
   if (g_test_verbose ())
-    g_print ("\t=> check global title/description...");
+    g_printerr ("\t=> check global title/description...");
   g_bookmark_file_set_title (bookmark, NULL, "a file");
   g_bookmark_file_set_description (bookmark, NULL, "a bookmark file");
 
@@ -345,19 +505,25 @@ test_modify (GBookmarkFile *bookmark)
   g_assert_cmpstr (text, ==, "a bookmark file");
   g_free (text);
   if (g_test_verbose ())
-    g_print ("ok\n");
+    g_printerr ("ok\n");
 
   if (g_test_verbose ())
-    g_print ("\t=> check bookmark title/description...");
+    g_printerr ("\t=> check bookmark title/description...");
   g_bookmark_file_set_title (bookmark, TEST_URI_0, "a title");
   g_bookmark_file_set_description (bookmark, TEST_URI_0, "a description");
   g_bookmark_file_set_is_private (bookmark, TEST_URI_0, TRUE);
-  time (&now);
-  g_bookmark_file_set_added (bookmark, TEST_URI_0, now);
-  g_bookmark_file_set_modified (bookmark, TEST_URI_0, now);
-  g_bookmark_file_set_visited (bookmark, TEST_URI_0, now);
+  now = g_date_time_new_now_utc ();
+  g_bookmark_file_set_added_date_time (bookmark, TEST_URI_0, now);
+  g_bookmark_file_set_visited_date_time (bookmark, TEST_URI_0, now);
   g_bookmark_file_set_icon (bookmark, TEST_URI_0, "testicon", "image/png");
 
+  /* Check the modification date by itself, as it’s updated whenever we modify
+   * other properties. */
+  g_bookmark_file_set_modified_date_time (bookmark, TEST_URI_0, now);
+  stamp = g_bookmark_file_get_modified_date_time (bookmark, TEST_URI_0, &error);
+  g_assert_no_error (error);
+  g_assert_cmpint (g_date_time_compare (stamp, now), ==, 0);
+
   text = g_bookmark_file_get_title (bookmark, TEST_URI_0, &error);
   g_assert_no_error (error);
   g_assert_cmpstr (text, ==, "a title");
@@ -366,64 +532,61 @@ test_modify (GBookmarkFile *bookmark)
   g_assert_no_error (error);
   g_assert_cmpstr (text, ==, "a description");
   g_free (text);
-  g_assert (g_bookmark_file_get_is_private (bookmark, TEST_URI_0, &error));
+  g_assert_true (g_bookmark_file_get_is_private (bookmark, TEST_URI_0, &error));
   g_assert_no_error (error);
-  stamp = g_bookmark_file_get_added (bookmark, TEST_URI_0, &error);
+  stamp = g_bookmark_file_get_added_date_time (bookmark, TEST_URI_0, &error);
   g_assert_no_error (error);
-  g_assert (stamp == now);
-  stamp = g_bookmark_file_get_modified (bookmark, TEST_URI_0, &error);
+  g_assert_cmpint (g_date_time_compare (stamp, now), ==, 0);
+  stamp = g_bookmark_file_get_visited_date_time (bookmark, TEST_URI_0, &error);
   g_assert_no_error (error);
-  g_assert (stamp == now);
-  stamp = g_bookmark_file_get_visited (bookmark, TEST_URI_0, &error);
-  g_assert_no_error (error);
-  g_assert (stamp == now);
-  g_assert (g_bookmark_file_get_icon (bookmark, TEST_URI_0, &icon, &mime, &error));
+  g_assert_cmpint (g_date_time_compare (stamp, now), ==, 0);
+  g_assert_true (g_bookmark_file_get_icon (bookmark, TEST_URI_0, &icon, &mime, &error));
   g_assert_no_error (error);
   g_assert_cmpstr (icon, ==, "testicon");
   g_assert_cmpstr (mime, ==, "image/png");
   g_free (icon);
   g_free (mime);
   if (g_test_verbose ())
-    g_print ("ok\n");
+    g_printerr ("ok\n");
 
   if (g_test_verbose ())
-    g_print ("\t=> check non existing bookmark...");
+    g_printerr ("\t=> check non existing bookmark...");
   g_bookmark_file_get_description (bookmark, TEST_URI_1, &error);
   g_assert_error (error, G_BOOKMARK_FILE_ERROR, G_BOOKMARK_FILE_ERROR_URI_NOT_FOUND);
   g_clear_error (&error);
   g_bookmark_file_get_is_private (bookmark, TEST_URI_1, &error);
   g_assert_error (error, G_BOOKMARK_FILE_ERROR, G_BOOKMARK_FILE_ERROR_URI_NOT_FOUND);
   g_clear_error (&error);
-  g_bookmark_file_get_added (bookmark, TEST_URI_1, &error);
+  g_bookmark_file_get_added_date_time (bookmark, TEST_URI_1, &error);
   g_assert_error (error, G_BOOKMARK_FILE_ERROR, G_BOOKMARK_FILE_ERROR_URI_NOT_FOUND);
   g_clear_error (&error);
-  g_bookmark_file_get_modified (bookmark, TEST_URI_1, &error);
+  g_bookmark_file_get_modified_date_time (bookmark, TEST_URI_1, &error);
   g_assert_error (error, G_BOOKMARK_FILE_ERROR, G_BOOKMARK_FILE_ERROR_URI_NOT_FOUND);
   g_clear_error (&error);
-  g_bookmark_file_get_visited (bookmark, TEST_URI_1, &error);
+  g_bookmark_file_get_visited_date_time (bookmark, TEST_URI_1, &error);
   g_assert_error (error, G_BOOKMARK_FILE_ERROR, G_BOOKMARK_FILE_ERROR_URI_NOT_FOUND);
   g_clear_error (&error);
   if (g_test_verbose ())
-    g_print ("ok\n");
+    g_printerr ("ok\n");
 
   if (g_test_verbose ())
-    g_print ("\t=> check application...");
+    g_printerr ("\t=> check application...");
   g_bookmark_file_set_mime_type (bookmark, TEST_URI_0, TEST_MIME);
-  g_assert (!g_bookmark_file_has_application (bookmark, TEST_URI_0, TEST_APP_NAME, NULL));
+  g_assert_false (g_bookmark_file_has_application (bookmark, TEST_URI_0, TEST_APP_NAME, NULL));
   g_bookmark_file_add_application (bookmark, TEST_URI_0,
 				   TEST_APP_NAME,
 				   TEST_APP_EXEC);
-  g_assert (g_bookmark_file_has_application (bookmark, TEST_URI_0, TEST_APP_NAME, NULL));
-  g_bookmark_file_get_app_info (bookmark, TEST_URI_0, TEST_APP_NAME,
-		  		&text,
-				&count,
-				&stamp,
-				&error);
+  g_assert_true (g_bookmark_file_has_application (bookmark, TEST_URI_0, TEST_APP_NAME, NULL));
+  g_bookmark_file_get_application_info (bookmark, TEST_URI_0, TEST_APP_NAME,
+                                        &text,
+                                        &count,
+                                        &stamp,
+                                        &error);
   g_assert_no_error (error);
-  g_assert (count == 1);
-  g_assert (stamp == g_bookmark_file_get_modified (bookmark, TEST_URI_0, NULL));
+  g_assert_cmpuint (count, ==, 1);
+  g_assert_cmpint (g_date_time_compare (stamp, g_bookmark_file_get_modified_date_time (bookmark, TEST_URI_0, NULL)), <=, 0);
   g_free (text);
-  g_assert (g_bookmark_file_remove_application (bookmark, TEST_URI_0, TEST_APP_NAME, &error));
+  g_assert_true (g_bookmark_file_remove_application (bookmark, TEST_URI_0, TEST_APP_NAME, &error));
   g_assert_no_error (error);
   g_bookmark_file_add_application (bookmark, TEST_URI_0, TEST_APP_NAME, TEST_APP_EXEC);
   apps = g_bookmark_file_get_applications (bookmark, TEST_URI_0, &length, &error);
@@ -432,24 +595,24 @@ test_modify (GBookmarkFile *bookmark)
   g_assert_cmpstr (apps[0], ==, TEST_APP_NAME);
   g_strfreev (apps);
 
-  g_bookmark_file_get_app_info (bookmark, TEST_URI_0, "fail",
-		  		&text,
-				&count,
-				&stamp,
-				&error);
+  g_bookmark_file_get_application_info (bookmark, TEST_URI_0, "fail",
+                                        &text,
+                                        &count,
+                                        &stamp,
+                                        &error);
   g_assert_error (error, G_BOOKMARK_FILE_ERROR, G_BOOKMARK_FILE_ERROR_APP_NOT_REGISTERED);
   g_clear_error (&error);
 
   if (g_test_verbose ())
-    g_print ("ok\n"); 
+    g_printerr ("ok\n");
 
   if (g_test_verbose ())
-    g_print ("\t=> check groups...");
-  g_assert (!g_bookmark_file_has_group (bookmark, TEST_URI_1, "Test", NULL));
+    g_printerr ("\t=> check groups...");
+  g_assert_false (g_bookmark_file_has_group (bookmark, TEST_URI_1, "Test", NULL));
   g_bookmark_file_add_group (bookmark, TEST_URI_1, "Test");
-  g_assert (g_bookmark_file_has_group (bookmark, TEST_URI_1, "Test", NULL));
-  g_assert (!g_bookmark_file_has_group (bookmark, TEST_URI_1, "Fail", NULL));
-  g_assert (g_bookmark_file_remove_group (bookmark, TEST_URI_1, "Test", &error));
+  g_assert_true (g_bookmark_file_has_group (bookmark, TEST_URI_1, "Test", NULL));
+  g_assert_false (g_bookmark_file_has_group (bookmark, TEST_URI_1, "Fail", NULL));
+  g_assert_true (g_bookmark_file_remove_group (bookmark, TEST_URI_1, "Test", &error));
   g_assert_no_error (error);
   groups = g_bookmark_file_get_groups (bookmark, TEST_URI_1, NULL, &error);
   g_assert_cmpint (g_strv_length (groups), ==, 0);
@@ -466,18 +629,20 @@ test_modify (GBookmarkFile *bookmark)
   g_assert_no_error (error);
 
   if (g_test_verbose ())
-    g_print ("ok\n");
+    g_printerr ("ok\n");
 
   if (g_test_verbose ())
-    g_print ("\t=> check remove...");
-  g_assert (g_bookmark_file_remove_item (bookmark, TEST_URI_1, &error) == TRUE);
+    g_printerr ("\t=> check remove...");
+  g_assert_true (g_bookmark_file_remove_item (bookmark, TEST_URI_1, &error));
   g_assert_no_error (error);
-  g_assert (g_bookmark_file_remove_item (bookmark, TEST_URI_1, &error) == FALSE);
+  g_assert_false (g_bookmark_file_remove_item (bookmark, TEST_URI_1, &error));
   g_assert_error (error, G_BOOKMARK_FILE_ERROR, G_BOOKMARK_FILE_ERROR_URI_NOT_FOUND);
   g_clear_error (&error);
   if (g_test_verbose ())
-    g_print ("ok\n");
-  
+    g_printerr ("ok\n");
+
+  g_date_time_unref (now);
+
   return TRUE;
 }
 
@@ -491,7 +656,7 @@ test_file (gconstpointer d)
   GError *error;
 
   bookmark_file = g_bookmark_file_new ();
-  g_assert (bookmark_file != NULL);
+  g_assert_nonnull (bookmark_file);
 
   success = test_load (bookmark_file, filename);
 
@@ -509,7 +674,7 @@ test_file (gconstpointer d)
 
   g_bookmark_file_free (bookmark_file);
 
-  g_assert (success == (strstr (filename, "fail") == NULL));
+  g_assert_true (success == (strstr (filename, "fail") == NULL));
 }
 
 int
@@ -520,7 +685,7 @@ main (int argc, char *argv[])
   const gchar *name;
   gchar *path;
 
-  g_test_init (&argc, &argv, NULL);
+  g_test_init (&argc, &argv, G_TEST_OPTION_ISOLATE_DIRS, NULL);
 
   if (argc > 1)
     {
@@ -532,6 +697,7 @@ main (int argc, char *argv[])
   g_test_add_func ("/bookmarks/to-file", test_to_file);
   g_test_add_func ("/bookmarks/move-item", test_move_item);
   g_test_add_func ("/bookmarks/misc", test_misc);
+  g_test_add_func ("/bookmarks/deprecated", test_deprecated);
 
   error = NULL;
   path = g_test_build_filename (G_TEST_DIST, "bookmarks", NULL);
