@@ -1,21 +1,31 @@
 /*
  * Copyright 2011 Collabora Ltd.
  *
- * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU Lesser General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * See the included COPYING file for more information.
  */
 
 #undef G_DISABLE_ASSERT
-#undef G_LOG_DOMAIN
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include "glib.h"
+#include "glib/gstrfuncsprivate.h"
+
+/* Keep in sync with glib/gbytes.c */
+struct _GBytes
+{
+  gconstpointer data;
+  gsize size;
+  gint ref_count;
+  GDestroyNotify free_func;
+  gpointer user_data;
+};
 
 static const gchar *NYAN = "nyannyan";
 static const gsize N_NYAN = 8;
@@ -33,7 +43,7 @@ test_new (void)
   g_assert (g_bytes_get_data (bytes, &size) != data);
   g_assert_cmpuint (size, ==, 4);
   g_assert_cmpuint (g_bytes_get_size (bytes), ==, 4);
-  g_assert (memcmp (data, g_bytes_get_data (bytes, NULL), g_bytes_get_size (bytes)) == 0);
+  g_assert_cmpmem (data, 4, g_bytes_get_data (bytes, NULL), g_bytes_get_size (bytes));
 
   g_bytes_unref (bytes);
 }
@@ -84,13 +94,65 @@ test_new_from_bytes (void)
 
   g_assert (sub != NULL);
   g_assert (g_bytes_get_data (sub, NULL) == ((gchar *)g_bytes_get_data (bytes, NULL)) + 10);
-  g_assert (g_bytes_get_size (sub) == 4);
   g_bytes_unref (bytes);
 
-  g_assert (memcmp (g_bytes_get_data (sub, NULL), "wave", 4) == 0);
+  g_assert_cmpmem (g_bytes_get_data (sub, NULL), g_bytes_get_size (sub), "wave", 4);
   g_bytes_unref (sub);
 }
 
+/* Verify that creating slices of GBytes reference the top-most bytes
+ * at the correct offset. Ensure that intermediate GBytes are not referenced.
+ */
+static void
+test_new_from_bytes_slice (void)
+{
+  GBytes *bytes = g_bytes_new_static ("Some stupid data", strlen ("Some stupid data") + 1);
+  GBytes *bytes1 = g_bytes_new_from_bytes (bytes, 4, 13);
+  GBytes *bytes2 = g_bytes_new_from_bytes (bytes1, 1, 12);
+  GBytes *bytes3 = g_bytes_new_from_bytes (bytes2, 0, 6);
+
+  g_assert_cmpint (bytes->ref_count, ==, 4);
+  g_assert_cmpint (bytes1->ref_count, ==, 1);
+  g_assert_cmpint (bytes2->ref_count, ==, 1);
+  g_assert_cmpint (bytes3->ref_count, ==, 1);
+
+  g_assert_null (bytes->user_data);
+  g_assert (bytes1->user_data == bytes);
+  g_assert (bytes2->user_data == bytes);
+  g_assert (bytes3->user_data == bytes);
+
+  g_assert_cmpint (17, ==, g_bytes_get_size (bytes));
+  g_assert_cmpint (13, ==, g_bytes_get_size (bytes1));
+  g_assert_cmpint (12, ==, g_bytes_get_size (bytes2));
+  g_assert_cmpint (6, ==, g_bytes_get_size (bytes3));
+
+  g_assert_cmpint (0, ==, strncmp ("Some stupid data", (gchar *)bytes->data, 17));
+  g_assert_cmpint (0, ==, strncmp (" stupid data", (gchar *)bytes1->data, 13));
+  g_assert_cmpint (0, ==, strncmp ("stupid data", (gchar *)bytes2->data, 12));
+  g_assert_cmpint (0, ==, strncmp ("stupid", (gchar *)bytes3->data, 6));
+
+  g_bytes_unref (bytes);
+  g_bytes_unref (bytes1);
+  g_bytes_unref (bytes2);
+  g_bytes_unref (bytes3);
+}
+
+/* Ensure that referencing an entire GBytes just returns the same bytes
+ * instance (with incremented reference count) instead of a new instance.
+ */
+static void
+test_new_from_bytes_shared_ref (void)
+{
+  GBytes *bytes = g_bytes_new_static ("Some data", strlen ("Some data") + 1);
+  GBytes *other = g_bytes_new_from_bytes (bytes, 0, g_bytes_get_size (bytes));
+
+  g_assert (bytes == other);
+  g_assert_cmpint (bytes->ref_count, ==, 2);
+
+  g_bytes_unref (bytes);
+  g_bytes_unref (other);
+}
+
 static void
 on_destroy_increment (gpointer data)
 {
@@ -212,8 +274,7 @@ test_to_data_transferred (void)
   memory = g_bytes_get_data (bytes, NULL);
   data = g_bytes_unref_to_data (bytes, &size);
   g_assert (data == memory);
-  g_assert_cmpuint (size, ==, N_NYAN);
-  g_assert (memcmp (data, NYAN, N_NYAN) == 0);
+  g_assert_cmpmem (data, size, NYAN, N_NYAN);
   g_free (data);
 }
 
@@ -231,8 +292,7 @@ test_to_data_two_refs (void)
   memory = g_bytes_get_data (bytes, NULL);
   data = g_bytes_unref_to_data (bytes, &size);
   g_assert (data != memory);
-  g_assert_cmpuint (size, ==, N_NYAN);
-  g_assert (memcmp (data, NYAN, N_NYAN) == 0);
+  g_assert_cmpmem (data, size, NYAN, N_NYAN);
   g_free (data);
   g_assert (g_bytes_get_data (bytes, &size) == memory);
   g_assert_cmpuint (size, ==, N_NYAN);
@@ -252,8 +312,7 @@ test_to_data_non_malloc (void)
   g_assert (g_bytes_get_data (bytes, NULL) == NYAN);
   data = g_bytes_unref_to_data (bytes, &size);
   g_assert (data != (gpointer)NYAN);
-  g_assert_cmpuint (size, ==, N_NYAN);
-  g_assert (memcmp (data, NYAN, N_NYAN) == 0);
+  g_assert_cmpmem (data, size, NYAN, N_NYAN);
   g_free (data);
 }
 
@@ -270,11 +329,42 @@ test_to_array_transferred (void)
   array = g_bytes_unref_to_array (bytes);
   g_assert (array != NULL);
   g_assert (array->data == memory);
-  g_assert_cmpuint (array->len, ==, N_NYAN);
-  g_assert (memcmp (array->data, NYAN, N_NYAN) == 0);
+  g_assert_cmpmem (array->data, array->len, NYAN, N_NYAN);
   g_byte_array_unref (array);
 }
 
+static void
+test_to_array_transferred_oversize (void)
+{
+  g_test_message ("g_bytes_unref_to_array() can only take GBytes up to "
+                  "G_MAXUINT in length; test that longer ones are rejected");
+
+  if (sizeof (guint) >= sizeof (gsize))
+    {
+      g_test_skip ("Skipping test as guint is not smaller than gsize");
+    }
+  else if (g_test_undefined ())
+    {
+      GByteArray *array = NULL;
+      GBytes *bytes = NULL;
+      gpointer data = g_memdup2 (NYAN, N_NYAN);
+      gsize len = ((gsize) G_MAXUINT) + 1;
+
+      bytes = g_bytes_new_take (data, len);
+      g_test_expect_message (G_LOG_DOMAIN, G_LOG_LEVEL_CRITICAL,
+                             "g_byte_array_new_take: assertion 'len <= G_MAXUINT' failed");
+      array = g_bytes_unref_to_array (g_steal_pointer (&bytes));
+      g_test_assert_expected_messages ();
+      g_assert_null (array);
+
+      g_free (data);
+    }
+  else
+    {
+      g_test_skip ("Skipping test as testing undefined behaviour is disabled");
+    }
+}
+
 static void
 test_to_array_two_refs (void)
 {
@@ -290,8 +380,7 @@ test_to_array_two_refs (void)
   array = g_bytes_unref_to_array (bytes);
   g_assert (array != NULL);
   g_assert (array->data != memory);
-  g_assert_cmpuint (array->len, ==, N_NYAN);
-  g_assert (memcmp (array->data, NYAN, N_NYAN) == 0);
+  g_assert_cmpmem (array->data, array->len, NYAN, N_NYAN);
   g_byte_array_unref (array);
   g_assert (g_bytes_get_data (bytes, &size) == memory);
   g_assert_cmpuint (size, ==, N_NYAN);
@@ -311,8 +400,7 @@ test_to_array_non_malloc (void)
   array = g_bytes_unref_to_array (bytes);
   g_assert (array != NULL);
   g_assert (array->data != (gpointer)NYAN);
-  g_assert_cmpuint (array->len, ==, N_NYAN);
-  g_assert (memcmp (array->data, NYAN, N_NYAN) == 0);
+  g_assert_cmpmem (array->data, array->len, NYAN, N_NYAN);
   g_byte_array_unref (array);
 }
 
@@ -336,20 +424,23 @@ main (int argc, char *argv[])
 {
   g_test_init (&argc, &argv, NULL);
 
-  g_test_bug_base ("http://bugs.gnome.org/");
+  g_test_bug_base ("https://bugzilla.gnome.org/");
 
   g_test_add_func ("/bytes/new", test_new);
   g_test_add_func ("/bytes/new-take", test_new_take);
   g_test_add_func ("/bytes/new-static", test_new_static);
   g_test_add_func ("/bytes/new-with-free-func", test_new_with_free_func);
   g_test_add_func ("/bytes/new-from-bytes", test_new_from_bytes);
+  g_test_add_func ("/bytes/new-from-bytes-slice", test_new_from_bytes_slice);
+  g_test_add_func ("/bytes/new-from-bytes-shared-ref", test_new_from_bytes_shared_ref);
   g_test_add_func ("/bytes/hash", test_hash);
   g_test_add_func ("/bytes/equal", test_equal);
   g_test_add_func ("/bytes/compare", test_compare);
-  g_test_add_func ("/bytes/to-data/transfered", test_to_data_transferred);
+  g_test_add_func ("/bytes/to-data/transferred", test_to_data_transferred);
   g_test_add_func ("/bytes/to-data/two-refs", test_to_data_two_refs);
   g_test_add_func ("/bytes/to-data/non-malloc", test_to_data_non_malloc);
-  g_test_add_func ("/bytes/to-array/transfered", test_to_array_transferred);
+  g_test_add_func ("/bytes/to-array/transferred", test_to_array_transferred);
+  g_test_add_func ("/bytes/to-array/transferred/oversize", test_to_array_transferred_oversize);
   g_test_add_func ("/bytes/to-array/two-refs", test_to_array_two_refs);
   g_test_add_func ("/bytes/to-array/non-malloc", test_to_array_non_malloc);
   g_test_add_func ("/bytes/null", test_null);
