@@ -722,7 +722,7 @@ random_test (gconstpointer d)
       if (qinf->head != q->head ||
           qinf->tail != q->tail ||
           qinf->length != q->length)
-        g_print ("op: %d\n", op);
+        g_printerr ("op: %d\n", op);
 
       g_assert (qinf->head == q->head);
       g_assert (qinf->tail == q->tail);
@@ -1058,6 +1058,161 @@ new_item (int x)
   return item;
 }
 
+static void
+test_clear_full (void)
+{
+  QueueItem *one, *two, *three, *four;
+  GQueue *queue;
+
+  queue = g_queue_new ();
+  g_queue_push_tail (queue, one = new_item (1));
+  g_queue_push_tail (queue, two = new_item (2));
+  g_queue_push_tail (queue, three = new_item (3));
+  g_queue_push_tail (queue, four = new_item (4));
+
+  g_assert_cmpint (g_queue_get_length (queue), ==, 4);
+  g_assert_false (one->freed);
+  g_assert_false (two->freed);
+  g_assert_false (three->freed);
+  g_assert_false (four->freed);
+
+  g_queue_clear_full (queue, free_func);
+
+  g_assert_true (one->freed);
+  g_assert_true (two->freed);
+  g_assert_true (three->freed);
+  g_assert_true (four->freed);
+
+  g_assert_true (g_queue_is_empty (queue));
+  check_integrity (queue);
+
+  g_slice_free (QueueItem, one);
+  g_slice_free (QueueItem, two);
+  g_slice_free (QueueItem, three);
+  g_slice_free (QueueItem, four);
+  g_queue_free (queue);
+}
+
+/* Check that g_queue_clear_full() called with a NULL free_func is equivalent
+ * to g_queue_clear(). */
+static void
+test_clear_full_noop (void)
+{
+  QueueItem *one, *two, *three, *four;
+  GQueue *queue;
+
+  queue = g_queue_new ();
+  g_queue_push_tail (queue, one = new_item (1));
+  g_queue_push_tail (queue, two = new_item (2));
+  g_queue_push_tail (queue, three = new_item (3));
+  g_queue_push_tail (queue, four = new_item (4));
+
+  g_assert_cmpint (g_queue_get_length (queue), ==, 4);
+  g_assert_false (one->freed);
+  g_assert_false (two->freed);
+  g_assert_false (three->freed);
+  g_assert_false (four->freed);
+
+  g_queue_clear_full (queue, NULL);
+
+  g_assert_true (g_queue_is_empty (queue));
+  check_integrity (queue);
+
+  g_slice_free (QueueItem, one);
+  g_slice_free (QueueItem, two);
+  g_slice_free (QueueItem, three);
+  g_slice_free (QueueItem, four);
+  g_queue_free (queue);
+}
+
+/* Test g_queue_push_nth_link() with various combinations of position (before,
+ * in the middle of, or at the end of the queue) and various existing queues
+ * (empty, single element, multiple elements). */
+static void
+test_push_nth_link (void)
+{
+  GQueue *q;
+  q = g_queue_new ();
+
+  /* Push onto before the front of an empty queue (which results in it being
+   * added to the end of the queue). */
+  g_queue_push_nth_link (q, -1, g_list_prepend (NULL, GINT_TO_POINTER (1)));
+  check_integrity (q);
+  g_assert_cmpint (g_queue_get_length (q), ==, 1);
+  g_assert_cmpint (GPOINTER_TO_INT (g_queue_peek_nth (q, 0)), ==, 1);
+
+  g_queue_clear (q);
+
+  /* Push onto after the rear of an empty queue. */
+  g_queue_push_nth_link (q, 100, g_list_prepend (NULL, GINT_TO_POINTER (2)));
+  check_integrity (q);
+  g_assert_cmpint (g_queue_get_length (q), ==, 1);
+  g_assert_cmpint (GPOINTER_TO_INT (g_queue_peek_nth (q, 0)), ==, 2);
+
+  g_queue_clear (q);
+
+  /* Push onto the front of an empty queue. */
+  g_queue_push_nth_link (q, 0, g_list_prepend (NULL, GINT_TO_POINTER (3)));
+  check_integrity (q);
+  g_assert_cmpint (g_queue_get_length (q), ==, 1);
+  g_assert_cmpint (GPOINTER_TO_INT (g_queue_peek_nth (q, 0)), ==, 3);
+
+  g_queue_clear (q);
+
+  /* Push onto before the front of a non-empty queue (which results in it being
+   * added to the end of the queue). */
+  g_queue_push_head (q, GINT_TO_POINTER (4));
+  g_queue_push_nth_link (q, -1, g_list_prepend (NULL, GINT_TO_POINTER (5)));
+  check_integrity (q);
+  g_assert_cmpint (g_queue_get_length (q), ==, 2);
+  g_assert_cmpint (GPOINTER_TO_INT (g_queue_peek_nth (q, 0)), ==, 4);
+  g_assert_cmpint (GPOINTER_TO_INT (g_queue_peek_nth (q, 1)), ==, 5);
+
+  g_queue_clear (q);
+
+  /* Push onto after the rear of a non-empty queue. */
+  g_queue_push_head (q, GINT_TO_POINTER (6));
+  g_queue_push_nth_link (q, 100, g_list_prepend (NULL, GINT_TO_POINTER (7)));
+  check_integrity (q);
+  g_assert_cmpint (g_queue_get_length (q), ==, 2);
+  g_assert_cmpint (GPOINTER_TO_INT (g_queue_peek_nth (q, 0)), ==, 6);
+  g_assert_cmpint (GPOINTER_TO_INT (g_queue_peek_nth (q, 1)), ==, 7);
+
+  g_queue_clear (q);
+
+  /* Push onto the rear of a non-empty queue. */
+  g_queue_push_head (q, GINT_TO_POINTER (8));
+  g_queue_push_nth_link (q, 1, g_list_prepend (NULL, GINT_TO_POINTER (9)));
+  check_integrity (q);
+  g_assert_cmpint (g_queue_get_length (q), ==, 2);
+  g_assert_cmpint (GPOINTER_TO_INT (g_queue_peek_nth (q, 0)), ==, 8);
+  g_assert_cmpint (GPOINTER_TO_INT (g_queue_peek_nth (q, 1)), ==, 9);
+
+  g_queue_clear (q);
+
+  /* Push onto the front of a non-empty queue. */
+  g_queue_push_head (q, GINT_TO_POINTER (10));
+  g_queue_push_nth_link (q, 0, g_list_prepend (NULL, GINT_TO_POINTER (11)));
+  check_integrity (q);
+  g_assert_cmpint (g_queue_get_length (q), ==, 2);
+  g_assert_cmpint (GPOINTER_TO_INT (g_queue_peek_nth (q, 0)), ==, 11);
+  g_assert_cmpint (GPOINTER_TO_INT (g_queue_peek_nth (q, 1)), ==, 10);
+
+  g_queue_clear (q);
+
+  /* Push into the middle of a non-empty queue. */
+  g_queue_push_head (q, GINT_TO_POINTER (12));
+  g_queue_push_head (q, GINT_TO_POINTER (13));
+  g_queue_push_nth_link (q, 1, g_list_prepend (NULL, GINT_TO_POINTER (14)));
+  check_integrity (q);
+  g_assert_cmpint (g_queue_get_length (q), ==, 3);
+  g_assert_cmpint (GPOINTER_TO_INT (g_queue_peek_nth (q, 0)), ==, 13);
+  g_assert_cmpint (GPOINTER_TO_INT (g_queue_peek_nth (q, 1)), ==, 14);
+  g_assert_cmpint (GPOINTER_TO_INT (g_queue_peek_nth (q, 2)), ==, 12);
+
+  g_queue_free (q);
+}
+
 static void
 test_free_full (void)
 {
@@ -1080,6 +1235,40 @@ test_free_full (void)
   g_slice_free (QueueItem, three);
 }
 
+static void
+test_insert_sibling_link (void)
+{
+  GQueue q = G_QUEUE_INIT;
+  GList a = {0};
+  GList b = {0};
+  GList c = {0};
+  GList d = {0};
+  GList e = {0};
+
+  g_queue_push_head_link (&q, &a);
+  g_queue_insert_after_link (&q, &a, &d);
+  g_queue_insert_before_link (&q, &d, &b);
+  g_queue_insert_after_link (&q, &b, &c);
+  g_queue_insert_after_link (&q, NULL, &e);
+
+  g_assert_true (q.head == &e);
+  g_assert_true (q.tail == &d);
+
+  g_assert_null (e.prev);
+  g_assert_true (e.next == &a);
+
+  g_assert_true (a.prev == &e);
+  g_assert_true (a.next == &b);
+
+  g_assert_true (b.prev == &a);
+  g_assert_true (b.next == &c);
+
+  g_assert_true (c.prev == &b);
+  g_assert_true (c.next == &d);
+
+  g_assert_true (d.prev == &c);
+  g_assert_null (d.next);
+}
 
 int main (int argc, char *argv[])
 {
@@ -1095,6 +1284,10 @@ int main (int argc, char *argv[])
   g_test_add_func ("/queue/static", test_static);
   g_test_add_func ("/queue/clear", test_clear);
   g_test_add_func ("/queue/free-full", test_free_full);
+  g_test_add_func ("/queue/clear-full", test_clear_full);
+  g_test_add_func ("/queue/clear-full/noop", test_clear_full_noop);
+  g_test_add_func ("/queue/insert-sibling-link", test_insert_sibling_link);
+  g_test_add_func ("/queue/push-nth-link", test_push_nth_link);
 
   seed = g_test_rand_int_range (0, G_MAXINT);
   path = g_strdup_printf ("/queue/random/seed:%u", seed);
