@@ -133,7 +133,7 @@ check_integrity (SequenceInfo *info)
 
 #if 0
   if (g_sequence_get_length (info->sequence) != info->n_items)
-    g_print ("%d %d\n",
+    g_printerr ("%d %d\n",
              g_sequence_get_length (info->sequence), info->n_items);
 #endif
   g_assert (info->n_items == g_queue_get_length (info->queue));
@@ -402,7 +402,7 @@ dump_info (SequenceInfo *seq)
   while (iter != g_sequence_get_end_iter (seq->sequence))
     {
       Item *item = get_item (iter);
-      g_print ("%p  %p    %d\n", list->data, iter, item->number);
+      g_printerr ("%p  %p    %d\n", list->data, iter, item->number);
 
       iter = g_sequence_iter_next (iter);
       list = list->next;
@@ -422,7 +422,7 @@ run_random_tests (gconstpointer d)
   int k;
 
 #if 0
-  g_print ("    seed: %u\n", seed);
+  g_printerr ("    seed: %u\n", seed);
 #endif
 
   g_random_set_seed (seed);
@@ -443,7 +443,7 @@ run_random_tests (gconstpointer d)
       int op = g_random_int_range (0, N_OPS);
 
 #if 0
-      g_print ("%d on %p\n", op, seq);
+      g_printerr ("%d on %p\n", op, seq);
 #endif
 
       switch (op)
@@ -1356,6 +1356,33 @@ test_stable_sort (void)
   g_sequence_free (seq);
 }
 
+static void
+test_empty (void)
+{
+  GSequence *seq;
+  int i;
+
+  seq = g_sequence_new (NULL);
+  g_assert_true (g_sequence_is_empty (seq));
+
+  for (i = 0; i < 1000; i++)
+    {
+      g_sequence_append (seq, GINT_TO_POINTER (i));
+      g_assert_false (g_sequence_is_empty (seq));
+    }
+
+  for (i = 0; i < 1000; i++)
+    {
+      GSequenceIter *end = g_sequence_get_end_iter (seq);
+      g_assert_false (g_sequence_is_empty (seq));
+      g_sequence_remove (g_sequence_iter_prev (end));
+    }
+
+  g_assert_true (g_sequence_is_empty (seq));
+
+  g_sequence_free (seq);
+}
+
 int
 main (int argc,
       char **argv)
@@ -1371,6 +1398,7 @@ main (int argc,
   g_test_add_func ("/sequence/iter-move", test_iter_move);
   g_test_add_func ("/sequence/insert-sorted-non-pointer", test_insert_sorted_non_pointer);
   g_test_add_func ("/sequence/stable-sort", test_stable_sort);
+  g_test_add_func ("/sequence/is_empty", test_empty);
 
   /* Regression tests */
   for (i = 0; i < G_N_ELEMENTS (seeds); ++i)
@@ -1388,4 +1416,3 @@ main (int argc,
 
   return g_test_run ();
 }
-
