@@ -19,7 +19,9 @@
  * if advised of the possibility of such damage.
  */
 
+#ifndef GLIB_DISABLE_DEPRECATION_WARNINGS
 #define GLIB_DISABLE_DEPRECATION_WARNINGS
+#endif
 
 #define _XOPEN_SOURCE 600
 #include <ctype.h>
@@ -32,6 +34,8 @@
 #include <string.h>
 #include "glib.h"
 
+#include "gstrfuncsprivate.h"
+
 #if defined (_MSC_VER) && (_MSC_VER <= 1800)
 #define isnan(x) _isnan(x)
 
@@ -219,6 +223,26 @@ test_memdup (void)
   g_free (str_dup);
 }
 
+/* Testing g_memdup2() function with various positive and negative cases */
+static void
+test_memdup2 (void)
+{
+  gchar *str_dup = NULL;
+  const gchar *str = "The quick brown fox jumps over the lazy dog";
+
+  /* Testing negative cases */
+  g_assert_null (g_memdup2 (NULL, 1024));
+  g_assert_null (g_memdup2 (str, 0));
+  g_assert_null (g_memdup2 (NULL, 0));
+
+  /* Testing normal usage cases */
+  str_dup = g_memdup2 (str, strlen (str) + 1);
+  g_assert_nonnull (str_dup);
+  g_assert_cmpstr (str, ==, str_dup);
+
+  g_free (str_dup);
+}
+
 /* Testing g_strpcpy() function with various positive and negative cases */
 static void
 test_stpcpy (void)
@@ -731,6 +755,22 @@ test_strcompress_strescape (void)
   g_assert_cmpstr (str, ==, "abc\\\"\b\f\n\r\t\v\003\177\234\313");
   g_free (str);
   g_free (tmp);
+
+  /* Unicode round trip */
+  str = g_strescape ("héllø there⸘", NULL);
+  g_assert_nonnull (str);
+  g_assert_cmpstr (str, ==, "h\\303\\251ll\\303\\270 there\\342\\270\\230");
+  tmp = g_strcompress (str);
+  g_assert_nonnull (tmp);
+  g_assert_cmpstr (tmp, ==, "héllø there⸘");
+  g_free (tmp);
+  g_free (str);
+
+  /* Test expanding invalid escapes */
+  str = g_strcompress ("\\11/ \\118 \\8aa \\19");
+  g_assert_nonnull (str);
+  g_assert_cmpstr (str, ==, "\t/ \t8 8aa \0019");
+  g_free (str);
 }
 
 /* Testing g_ascii_strcasecmp() and g_ascii_strncasecmp() */
@@ -2523,6 +2563,7 @@ main (int   argc,
   g_test_add_func ("/strfuncs/has-prefix", test_has_prefix);
   g_test_add_func ("/strfuncs/has-suffix", test_has_suffix);
   g_test_add_func ("/strfuncs/memdup", test_memdup);
+  g_test_add_func ("/strfuncs/memdup2", test_memdup2);
   g_test_add_func ("/strfuncs/stpcpy", test_stpcpy);
   g_test_add_func ("/strfuncs/str_match_string", test_str_match_string);
   g_test_add_func ("/strfuncs/str_tokenize_and_fold", test_str_tokenize_and_fold);
