@@ -25,7 +25,7 @@
 #define DEFAULT_TEST_TIME 15 /* seconds */
  /* The time we want each round to take, in seconds, this should
   * be large enough compared to the timer resolution, but small
-  * enought that the risk of any random slowness will miss the
+  * enough that the risk of any random slowness will miss the
   * running window */
 #define TARGET_ROUND_TIME 0.008
 
@@ -125,6 +125,9 @@ run_test (PerformanceTest *test)
     g_print ("Running %"G_GINT64_MODIFIER"d rounds\n", num_rounds);
 
   /* Run the test */
+  avg_elapsed = 0.0;
+  min_elapsed = 0.0;
+  max_elapsed = 0.0;
   for (i = 0; i < num_rounds; i++)
     {
       test->init (test, data, factor);
@@ -144,7 +147,8 @@ run_test (PerformanceTest *test)
         }
     }
 
-  avg_elapsed = avg_elapsed / num_rounds;
+  if (num_rounds > 1)
+    avg_elapsed = avg_elapsed / num_rounds;
 
   if (verbose)
     {
@@ -152,6 +156,7 @@ run_test (PerformanceTest *test)
       g_print ("Maximum corrected round time: %.2f msecs\n", max_elapsed * 1000);
       g_print ("Average corrected round time: %.2f msecs\n", avg_elapsed * 1000);
     }
+
   /* Print the results */
   test->print_result (test, data, min_elapsed);
 
